/*
 * ext/Xeg/Xeg.c
 * 
 * by Nakamura Noritsugu
 * from 1997.09
 */

#include "ruby.h"
#include "XeasyGraphic.h"

static VALUE mXeg, cImage;

static float color[3];

static void set_color(struct RArray *color_p)
{
  int i, n;

  n = 3;
  for(i = 0; i <n; i++){
	color[i] = 0;
  }
  if(color_p->len < n) {
	n = color_p->len;
  }
  for(i = 0; i < n; i++){
/*	color[i] = NUM2INT(color_p->ptr[i]);*/
	if(TYPE(color_p->ptr[i]) == T_FLOAT) {
	  color[i] = RFLOAT(color_p->ptr[i])->value;
	} else {
	  color[i] = (double)NUM2INT(color_p->ptr[i]);
	}
  }
}

static void
free_img(id win)
{
  if (win) {
	GrWindowClose(win);
  }
}

static VALUE
img_s_new(VALUE class,int x, int y, int w, int h)
{
  id win;

  win = GrWindow(NUM2INT(x), NUM2INT(y), NUM2INT(w), NUM2INT(h));
  return Data_Wrap_Struct(class, 0, free_img, (id*) win);
}

static VALUE
img_line(VALUE win, int x1, int y1, int x2, int y2,
		 int lw, int colorId, struct RArray *color_p)
{
  id w;

  Data_Get_Struct(win, id, (id*) w);

  set_color(color_p);
  GrLine(w, NUM2INT(x1), NUM2INT(y1), NUM2INT(x2), NUM2INT(y2),
		 NUM2INT(lw), NUM2INT(colorId), color);
}

static VALUE
img_circle(VALUE win, int x, int y, int r,
		   int lw, int colorId, struct RArray *color_p)
{
  id w;

  Data_Get_Struct(win, id, (id*) w);

  set_color(color_p);
  GrCircle(w, NUM2INT(x), NUM2INT(y), NUM2INT(r), NUM2INT(lw), NUM2INT(colorId), color);
}

static VALUE
img_circle_fill(VALUE win, int x, int y, int r,
				int colorId, struct RArray *color_p)
{
  id w;

  Data_Get_Struct(win, id, (id*) w);

  set_color(color_p);
  GrCircleFill(w, NUM2INT(x), NUM2INT(y), NUM2INT(r), NUM2INT(colorId), color);
}

static VALUE
img_arc(VALUE win, int x, int y, int r, int t1, int t2,
		int lw, int colorId, struct RArray *color_p)
{
  id w;

  Data_Get_Struct(win, id, (id*) w);

  set_color(color_p);
  GrArc(w, NUM2INT(x), NUM2INT(y), NUM2INT(r), NUM2INT(t1), NUM2INT(t2),
		NUM2INT(lw), NUM2INT(colorId), color);
}

static VALUE
img_fan(VALUE win, int x, int y, int r, int t1, int t2,
		int lw, int colorId, struct RArray *color_p)
{
  id w;

  Data_Get_Struct(win, id, (id*) w);

  set_color(color_p);
  GrFan(w, NUM2INT(x), NUM2INT(y), NUM2INT(r), NUM2INT(t1), NUM2INT(t2),
		NUM2INT(lw), NUM2INT(colorId), color);
}

static VALUE
img_fan_fill(VALUE win, int x, int y, int r, int t1, int t2,
			 int colorId, struct RArray *color_p)
{
  id w;

  Data_Get_Struct(win, id, (id*) w);

  set_color(color_p);
  GrFanFill(w, NUM2INT(x), NUM2INT(y), NUM2INT(r),
			NUM2INT(t1), NUM2INT(t2), NUM2INT(colorId), color);
}

static VALUE
img_box(VALUE win, int x, int y, int w, int h,
		int lw, int colorId, struct RArray *color_p)
{
  id ww;

  Data_Get_Struct(win, id, (id*) ww);

  set_color(color_p);
  GrBox(ww, NUM2INT(x), NUM2INT(y), NUM2INT(w), NUM2INT(h),
		NUM2INT(lw), NUM2INT(colorId), color);
}

static VALUE
img_box_fill(VALUE win, int x, int y, int w, int h,
			 int colorId, struct RArray *color_p)
{
  id ww;

  Data_Get_Struct(win, id, (id*) ww);

  set_color(color_p);
  GrBoxFill(ww, NUM2INT(x), NUM2INT(y), NUM2INT(w),
			NUM2INT(h), NUM2INT(colorId), color);
}

static VALUE
img_polygon(VALUE win, struct RArray *x, struct RArray *y,
			int lw, int colorId, struct RArray *color_p)
{
  id w;
  int i, n;
  float *xx, *yy;

  Data_Get_Struct(win, id, (int*) w);

  set_color(color_p);

  n = x->len;
  if(n > y->len){
	n = y->len;
  }
/*  printf("n: %d\n", n);*/
  xx = (float *)xmalloc(sizeof(float) * n);
  yy = (float *)xmalloc(sizeof(float) * n);
  for(i = 0; i < n; i++){
	if(TYPE(x->ptr[i]) == T_FLOAT) {
	  *(xx + i) = RFLOAT(x->ptr[i])->value;
	} else {
	  *(xx + i) = (double)NUM2INT(x->ptr[i]);
	}
	if(TYPE(y->ptr[i]) == T_FLOAT) {
	  *(yy + i) = RFLOAT(y->ptr[i])->value;
	} else {
	  *(yy + i) = (double)NUM2INT(y->ptr[i]);
	}
  }
  GrPolygon(w, n, xx, yy, NUM2INT(lw), NUM2INT(colorId), color);
  free(xx); free(yy);
}

static VALUE
img_polygon_fill(VALUE win, struct RArray *x, struct RArray *y,
				 int colorId, struct RArray *color_p)
{
  id w;
  int i, n;
  float *xx, *yy;

  Data_Get_Struct(win, id, (int*) w);

  set_color(color_p);

  n = x->len;
  if(n > y->len){
	n = y->len;
  }
  xx = (float *)xmalloc(sizeof(float) * n);
  yy = (float *)xmalloc(sizeof(float) * n);
  for(i = 0; i < n; i++){
	if(TYPE(x->ptr[i]) == T_FLOAT) {
	  *(xx + i) = RFLOAT(x->ptr[i])->value;
	} else {
	  *(xx + i) = (double)NUM2INT(x->ptr[i]);
	}
	if(TYPE(y->ptr[i]) == T_FLOAT) {
	  *(yy + i) = RFLOAT(y->ptr[i])->value;
	} else {
	  *(yy + i) = (double)NUM2INT(y->ptr[i]);
	}
  }
  GrPolygonFill(w, n, xx, yy, NUM2INT(colorId), color);
  free(xx); free(yy);
}

static VALUE
img_text(VALUE win, int x, int y, struct RString *txt,
		 int type, int size, int colorId, struct RArray *color_p)
{
  id w;

  Data_Get_Struct(win, id, (int*) w);

  set_color(color_p);
  GrText(w, NUM2INT(x), NUM2INT(y), txt->ptr,
		 NUM2INT(type), NUM2INT(size), NUM2INT(colorId), color);
}

static VALUE
img_epsout(VALUE win, struct RString *filename)
{
  id w;

  Data_Get_Struct(win, id, (int*) w);

  GrEPSOut(w, filename->ptr);
}

static VALUE
img_window_clear(VALUE win)
{
  id w;

  Data_Get_Struct(win, id, (int*) w);

  GrWindowClear(w);
}

void
Init_Xeg()
{
    mXeg = rb_define_module("Xeg");
	cImage = rb_define_class_under(mXeg, "Image", cObject);
    rb_define_singleton_method(cImage, "new", img_s_new, 4);
    rb_define_method(cImage, "Line", img_line, 7);
    rb_define_method(cImage, "Circle", img_circle, 6);
    rb_define_method(cImage, "CircleFill", img_circle_fill, 5);
    rb_define_method(cImage, "Arc", img_arc, 8);
    rb_define_method(cImage, "Fan", img_fan, 8);
    rb_define_method(cImage, "FanFill", img_fan_fill, 7);
    rb_define_method(cImage, "Box", img_box, 7);
    rb_define_method(cImage, "BoxFill", img_box_fill, 6);
    rb_define_method(cImage, "Polygon", img_polygon, 5);
    rb_define_method(cImage, "PolygonFill", img_polygon_fill, 4);
    rb_define_method(cImage, "Text", img_text, 7);
    rb_define_method(cImage, "EPSOut", img_epsout, 1);
    rb_define_method(cImage, "Clear", img_window_clear, 0);

	rb_define_const(mXeg, "Gr_MONO", INT2FIX(Gr_MONO));
	rb_define_const(mXeg, "Gr_RGB",  INT2FIX(Gr_RGB));

	rb_define_const(mXeg, "Gr_C",  INT2FIX(Gr_C));
	rb_define_const(mXeg, "Gr_CB", INT2FIX(Gr_CB));
	rb_define_const(mXeg, "Gr_CO", INT2FIX(Gr_CO));
	rb_define_const(mXeg, "Gr_CD", INT2FIX(Gr_CD));
	rb_define_const(mXeg, "Gr_H",  INT2FIX(Gr_H));
	rb_define_const(mXeg, "Gr_HB", INT2FIX(Gr_HB));
	rb_define_const(mXeg, "Gr_HO", INT2FIX(Gr_HO));
	rb_define_const(mXeg, "Gr_HD", INT2FIX(Gr_HD));
	rb_define_const(mXeg, "Gr_TR", INT2FIX(Gr_TR));
	rb_define_const(mXeg, "Gr_TB", INT2FIX(Gr_TB));
	rb_define_const(mXeg, "Gr_TI", INT2FIX(Gr_TI));
	rb_define_const(mXeg, "Gr_TD", INT2FIX(Gr_TD));
	rb_define_const(mXeg, "Gr_S",  INT2FIX(Gr_S));
}
